norm.HMM.mle <-
function(x,m,sigma0=NA,gamma0=NA,...)
{ 

 if(is.na(gamma0[1]))
  gamma0 = matrix(rep(1,m*m),ncol=m) * 0.1/m + 0.9 * diag(m)

 if(is.na(sigma0[1])){
  N=5
  sd_max   = 3 * sd(x)
  #temp_seq = seq(from=0,to=sd_max,by=sd_max/(m+1))
  #sigma0=temp_seq[2:(m+1)]
  temp_werte=runif(m*10,min=sd_max/10, max= sd_max)
  sigma0_mat = matrix(temp_werte,ncol=m,byrow=T)
  parvect0_mat = matrix(rep(0,N*(m*(m-1)+m)),ncol=(m*(m-1)+m),byrow=T)
  for(i in 1:N) parvect0_mat[i,] = norm.HMM.pn2pw(m,sigma0_mat[i,],gamma0) 
 }                                                     
 else{
  parvect0  <- norm.HMM.pn2pw(m,sigma0,gamma0)
 }
 
 #---------------------------------------------------------
 #Function: norm.HMM.mllk()
 #Description: computes the negative log-likelihood of a Normal-HMM for a given set of working parameters
 #Arguments: 
 #parvect...working parameters (corresponding to the transition probability matrix (in the folling we write tpm) of the hidden
 # Makov chain and the state dependent variances)
 #x...sample of length n
 #m...number of states of the hidden Markov Chain
 #Value:
 #negative log likelihood
 #--------------------------------------------------------
 norm.HMM.mllk <- function(parvect,x,m,...){                                                  
 if(m==1) return(-sum(log(dnorm.scale(x,exp(parvect))))) 
 n          <- length(x)                            
 pn         <- norm.HMM.pw2pn(m,parvect)            
 allprobs   <- outer(x,pn$sigma,dnorm.scale)             
 allprobs   <- ifelse(!is.na(allprobs),allprobs,1)  
 lscale     <- 0                                    
 foo        <- pn$delta                             
 for (i in 1:n)                                    
   {                                                
   foo    <- foo%*%pn$gamma*allprobs[i,]            
   sumfoo <- sum(foo)                               
   lscale <- lscale+log(sumfoo)                    
   foo    <- foo/sumfoo                            
   }                                               
 mllk       <- -lscale                            
 mllk                                              
 } 
 
 
 if(is.na(sigma0[1])){
  mod_list=list(0)
  values=0
 
  for (i in 1:N){
    mod_list[[i]] = nlm(norm.HMM.mllk,parvect0_mat[i,],x=x,m=m) 
    values[i] = mod_list[[i]]$minimum
    print(i)
  }
 
  j0 = which.min(values)
  print(sigma0_mat[j0,])
  mod= mod_list[[j0]]
 }
 else{         
  mod       <- nlm(norm.HMM.mllk,parvect0,x=x,m=m) 
 }      
 pn        <- norm.HMM.pw2pn(m,mod$estimate)            
 mllk      <- mod$minimum                              
 np        <- (m*(m-1)+m) #length(parvect0)                          
 AIC       <- 2*(mllk+np)                              
 n         <- sum(!is.na(x))                            
 BIC       <- 2*mllk+np*log(n)                         
 list(sigma=pn$sigma,gamma=pn$gamma,delta=pn$delta,   
              code=mod$code,mllk=mllk,AIC=AIC,BIC=BIC)   
}

