\name{norm.HMM.generate_sample}
\alias{norm.HMM.generate_sample}
\title{
Generates a realization of a Normal Hidden Markov Model
}
\description{
This function generates a realization of a normal HMM with m states, where the state dependent distributions have a fixed mean of zero and switching standard deviations. The returned value is a list with the observable and the non-observable process.
}
\usage{
norm.HMM.generate_sample(n, m, sigma, gamma, delta = NULL)
}
\arguments{
  \item{n}{
length of the sample to be generated
}
  \item{m}{
number of states of the hidden Markov chain
}
  \item{sigma}{
m component vector with standard deviations
}
  \item{gamma}{
transition probability matrix of the hidden Markov chain
}
  \item{delta}{
initial distribution of the hidden Markov chain (DEFAULT: stationary distribution of the hidden Markov chain)
}
}

\value{
The returned value is a list with components:
  \item{x}{realisations of the observable process (length n)}
  \item{state}{realisations of the non-observable process (length n)}
}

\examples{
#set tpm
gamma=matrix(ncol=3,nrow=3)
gamma[1,1]=0.6
gamma[1,2]=0.1
gamma[1,3]=1-sum(gamma[1,1:2])
gamma[2,1]=0.1
gamma[2,2]=0.8
gamma[2,3]=1-sum(gamma[2,1:2])
gamma[3,1]=0.2
gamma[3,2]=0.05
gamma[3,3]=1-sum(gamma[3,1:2])
gamma

#set length
n=1000

#set number of states
m=3

#set regime dependant standard deviations
sigma=c(0.2,1,10) 

#genrate realization
HMM=norm.HMM.generate_sample(n,m,sigma,gamma)

y=HMM$x
s=HMM$state
}